%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% DemoEquivEllipse
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Overview:
%   Reconstruct the equivalent ellipse from the PT
%
% References:
%   Section 2.5.7
%   Mathematical and Computational Methods in Photonics and Phononics
%
% Authors:
%   Habib Ammari, Brian Fitzpatrick, Hyeonbae Kang,
%   Matias Ruiz, Sanghyeon Yu, Hai Zhang
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc, clear all, close all
% --------------------------------------------------
% Two circular unit disks separated by a distance 0.3.
% --------------------------------------------------
R0 = 1; 
dist = 0.3;  
angle = pi/6;

nbPoints=200;
d_center = dist/2+R0;
D1 = ellipse(R0,R0,-d_center*cos(angle),-d_center*sin(angle),nbPoints);
D2 = ellipse(R0,R0,d_center*cos(angle),d_center*sin(angle),nbPoints);

% --------------------------------------------------
% Compute PT.
% --------------------------------------------------
lambda = 3;
PT=getPT_2incl(D1,D2,lambda);

% --------------------------------------------------
% Reconstruct parameters for equivalent ellipse.
% --------------------------------------------------
[v d] = eig(PT); % d: eigenvalues of PT, v: eigenvectors of PT

k=(2*lambda+1)/(2*lambda-1);
e=(d(2,2)-k*d(1,1))/(d(1,1)-k*d(2,2));
AreaE = d(1,1)*(e+k)/(e+1)/(k-1); 
b = sqrt(AreaE/pi/e); 
a=e*b;

angle1 = atan(-v(1,2)/v(1,1)); 
if (angle1 < 0)
    angle1 = angle1+pi;
end
equiv_ellipse_temp = ellipse(a,b,0,0,nbPoints);
Dre = [cos(angle1) -sin(angle1); sin(angle1) cos(angle1)]*equiv_ellipse_temp.points;

fprintf('\n \nThe Polarization Tensor for two circular disks is\n')
disp(PT)
fprintf('\nParameters for the equivalent ellipse \n  a: %f \n  b: %f \n  angle: %f \n',a,b,angle1);

% --------------------------------------------------
% Plot two circular disks and their equivalent ellipse.
% --------------------------------------------------
figure(1)
clf
plot([D1.points(1,:) D1.points(1,1)], [D1.points(2,:) D1.points(2,1)],'linewidth',2, 'color',[0.50196     0.50196     0.50196] );
hold on
plot([D2.points(1,:) D2.points(1,1)], [D2.points(2,:) D2.points(2,1)],'linewidth',2, 'color',[0.50196     0.50196     0.50196] );
hold on
plot([Dre(1,:) Dre(1,1)], [Dre(2,:) Dre(2,1)],'linewidth',2,'color',[0    0     0] );
axis([-2.5,2.5,-2.5,2.5])
axis square
set(gca,'xtick',[-2 -1 0 1 2], 'fontsize',15)
set(gca,'ytick',[-2 -1 0 1 2], 'fontsize',15)
set(gca, 'Position', get(gca, 'OuterPosition') - get(gca, 'TightInset') * [-1 0 1 0; 0 -1 0 1; 0 0 1 0; 0 0 0 1]);
